program conred36;
{$F+}
{
***********************************************************
"Puente de tiristores. comando ARCO-COSENO"
Autores: JRomero - Arodri
Fecha: 18-04-98

		 La simulacin corresponde a un puente de tiristores
		 con iL=Id , y se obeservarn los siguientes
		 parmetros:

		 * tensines de entrada (todas las de  linea), contrastada con la de salida
		  tomando como carga una fuente de corriente.
		 * tensin auxiliar para el disparo de t1 contrastada con la uref
		 * corrientes de cada tiristor.

		 Se podr afectar el ngulo de disparo  mediante uref y
		 el valor del mismo estar acotado entre -0.99 y  0.99. El
		 recorrido se har a pasos de 0.03.

		 OBS: para implementar el disparo por el control arcocoseno
		 se toma como referencia a la fase R y se supone que esta es
		  sin(w*t).
**********************************************************
}

uses
wincrt, xmatdefs, FilUSR, UCntrl, Simul4, traxp, variable,
lib_basi, libep02, libep, libdig01, dibus;

const w=PI*100;           {frecuencia de red=50 hz}
		sexrad=PI/180;      {constante de conversin de sexagesimal a rad}
		periodo=20e-3;      {periodo=2*PI/w}
		grillay=10;          {divisiones de la grilla en y}
		grillax=12;			  {divisiones de la grilla en x}
		nper=1;				  {nmero de periodos a visualizar   (nper>=1)}
		rshunt=0.001;		  {valor de resitencias de medicin de corriente}

var   uref,alpha,wt:NReal;

(*********************************************************************
  Funcin para el disparo del tiristor.
*********************************************************************)
{
	El parmetro ALPHA es tomado como control del disparo del tiristor.
}

function disparo(k:integer): boolean;
var vref:NREAL;
begin
	vref:=-cos(alpha*sexrad);
	disparo:=(vref <= sin(w*t - 2*PI/3 - (k-1) * PI/3)) and (cos(w*t - 2*PI/3 - (k-1) * PI/3)>=0);
end;

(***********************************************************************
	Funciones de definicin del circuito

	Cirt1: componentes no booleanos
	Cirt2: componentes booleanos
***********************************************************************)

procedure cirt1;
begin
vg(1,11,1,311);							{generador R de 220 Vrms}
vg(2,11,2,311);							{generador T de 220 Vrms}
vg(3,11,3,311);							{generador S de 220 Vrms}


{resistencias de medicin de corrientes}
r(1,4,rshunt);
r(2,5,rshunt);
r(3,6,rshunt);
r(1,8,rshunt);
r(2,9,rshunt);
r(3,10,rshunt);

ig(0,7,0,10);								{fuente de corriente Id=1 A}

end;

procedure cirt2;
begin
thmi(4,7,disparo(1));							{Tiristor 1}
thmi(0,10,disparo(2));							{Tiristor 2}
thmi(5,7,disparo(3));							{Tiristor 3}
thmi(0,8,disparo(4));							{Tiristor 4}
thmi(6,7,disparo(5));							{Tiristor 5}
thmi(0,9,disparo(6));							{Tiristor 6}
end;


(************************************************************************
	Definicin de la seal del generador VG[1]
************************************************************************)
procedure funcsr(t:NReal);
begin
rf[1]:=sin(w*t);				{R}
rf[2]:=sin(w*t-2*PI/3);    {S}
rf[3]:=sin(w*t-4*PI/3);    {T}
end;




(************************************************************************
	Procedimientos necesarios de Simeep:

	inicio:	inicializacin de simulaciones, def. de pantalla,etc
	pesca: 	captura e impresin en pantalla de variables observadas
	fin:     procedimiento de finalizacin

************************************************************************)

procedure inicio;
begin
	nn:=11;					{nmero de nodos, sin incluir tierra (0)}
	nindep:=3;     		{nmero de funciones definidas en funcsr = numero de generadores}
	tinicial:=0;			{tiempo inicial de la simulacin}
	tfinal:=nper*periodo;	{tiempo final de la simulacin}


	{parmetros de simulacin}
	dtmax0:=5e-6;
	dtmin:=dtmax0/100;
	PasoDivisor:=6;
	TmaxTmin:=1e3;

	{inicializacin de la unidad de grficos}
	traxp.tinicial:=tinicial;
	traxp.tfinal:=tfinal;

	{inicializacin del modo grfico}
	iniciegr;

	{grillas en pantalla}
	gridx:=grillax;
	gridy:=grillay;

	{matriz de visualizacin de 1 por 4}
	subplot(1,3);


	{primera pantalla}
	definaY(0,-750,150);
	titulo('uD(t) (amarillo) y tensiones de linea: +-RS(rojo),+-ST(negro),+-TR(azul)');
	xlabel('30/DIV');
	ylabel('150 V/DIV');
	linea0;
	grid;

	gridy:=13;
	{segunda pantalla}
	definaY(1,0,10);
	titulo('corriente de tiristores');
	xlabel('30 /DIV');
	ylabel('  6  5  4  3   2  1');
	grid;

	gridy:=grillay;

	{tercera pantalla}
	definaY(2,-1.25,0.25);
	titulo('uref(amarillo) y tensiones auxiliares de disparo (rojo para T1)');
	xlabel('30 /DIV');
	ylabel('0.25 V/DIV');
	linea0;
	grid;

	{canal 4,5,6 en la misma pantalla que el canal 0}
	superponga(3,0);
	superponga(4,0);
	superponga(5,0);
	superponga(11,0);
	superponga(12,0);
	superponga(13,0);

	{canal 7,8,9,10,11,12 en la misma pantalla que el canal 2}
	superponga(6,1);
	superponga(7,1);
	superponga(8,1);
	superponga(9,1);
	superponga(10,1);

	{tensin de faseR con iR}
	superponga(14,2);
	superponga(15,2);
	superponga(16,2);
	superponga(17,2);
	superponga(18,2);
	superponga(19,2);
	superponga(20,2);
	superponga(21,2);
	superponga(22,2);

	{colores a mostrar por canal}

	{primera pantalla}
	definacolor(0,amarillo);			{uD}
	definacolor(3,rojo);			{RS}
	definacolor(4,negro);      {ST}
	definacolor(5,azul);       {TR}
	definacolor(11,rojo);		{SR}
	definacolor(12,negro);     {TS}
	definacolor(13,azul);      {RT}

	{segunda pantalla}
	definacolor(1,rojo);		{iT1}
	definacolor(6,8);			{iT2}
	definacolor(7,8);			{iT3}
	definacolor(8,8);		{iT4}
	definacolor(9,8);		{iT5}
	definacolor(10,8);		{iT6}

	{tercera pantalla}
	definacolor(2,amarillo);			{iR}
	definacolor(14,rojo);			{uR}
	definacolor(15,negro);			{uR}
	definacolor(16,negro);			{uR}
	definacolor(17,negro);			{uR}
	definacolor(18,negro);			{uR}
	definacolor(19,negro);			{uR}
	definacolor(20,negro);			{uR}
	definacolor(21,negro);			{uR}
	definacolor(22,negro);			{uR}

end;

{____________________________________________________________________}

procedure fin;
begin
TermineGr;
halt;
end;

{____________________________________________________________________}

procedure pesca;

var   k,enx:integer;
		grados:NReal;
begin

	{valor para alpha}
	if uref=0 then
		alpha:=90
	else
		if uref>0 then
			alpha:= ArcTan (sqrt (1-sqr (uref)) /uref) / sexrad
		else
			alpha:=180 - ArcTan (sqrt (1-sqr (-uref)) / -uref) / sexrad;




	traxp.t:=t;					{indicamos a traxp el tiempo de simulacin}
	wt:=w/sexrad*t;


	{*etiquetas de grficas*}
	grados:=(w/sexrad)*t;
	enX:=PUNTOX+21; {paso la ordenada x que pinta la onda a la variable enX por medio de la funcion puntox}
	{ANOTAR_EVENTO(numeracin, GRAD1,GRAD2,GRAD, ENX, y,y1a,y1b,y2a,y2b,etiqueta)}
	{alpha}ANOTAR_EVENTO(1,30,30+alpha,grados,enx,30,20,140,20,140, CHR(224)+' para T1');
	{ANOTAR_EVENTO(numeracin, GRAD1,GRAD2,GRAD, ENX, y,y1a,y1b,y2a,y2b,etiqueta)}
	{alpha}ANOTAR_EVENTO(2,30,30+alpha,grados,enx,345,338,458,338,458, CHR(224));


	{captura de valores}

	{primera pantalla}
	trazo(3,v[1]-v[2]);			{RS}
	trazo(4,v[2]-v[3]);      {ST}
	trazo(5,v[3]-v[1]);       {TR}
	trazo(11,v[2]-v[1]);			{SR}
	trazo(12,v[3]-v[2]);      {TS}
	trazo(13,v[1]-v[3]);       {RT}
	trazo(0,v[7]+20);			{uD}
	trazo(0,v[7]-20);			{uD}

	{segunda pantalla}
	trazo(1,(v[1]-v[4])*(1/rshunt)+110);		{iT1}
	trazo(6,(v[10]-v[3])*(1/rshunt)+90);			{iT2}
	trazo(7,(v[2]-v[5])*(1/rshunt)+70);			{iT3}
	trazo(8,(v[8]-v[1])*(1/rshunt)+50);		{iT4}
	trazo(9,(v[3]-v[6])*(1/rshunt)+30);		{iT5}
	trazo(10,(v[9]-v[2])*(1/rshunt)+10);		{iT6}

	trazo(1,110);		{iT1}
	trazo(6,90);			{iT2}
	trazo(7,70);			{iT3}
	trazo(8,50);		{iT4}
	trazo(9,30);		{iT5}
	trazo(10,10);		{iT6}


	{tercera pantalla}
		{cambio de colores para indicar el color rojo mientras se considera
		 la seal para efetuar la comparacin}
		if (t>1.6666666666e-3) and (t<7*1.66666666e-3) then
		begin
			trazo(14,-rf[2]+0.005);
			trazo(14,-rf[2]-0.005);
		end;

		if (t>3*1.6666666666e-3) and (t<9*1.66666666e-3) then
		begin
			trazo(15,rf[1]);
		end;


		if (t>5*1.6666666666e-3) and (t<11*1.66666666e-3) then
		begin
			trazo(16,-rf[3]);
		end;

		{los  2 siguientes corresponden a disparo de T4}
		if (t<1.6666666666e-3) then
		begin
			trazo(17,rf[2]);
		end;

		if (t>7*1.66666666e-3) then
		begin
			trazo(20,rf[2]);
		end;

		{los 2 siguientes corresponden a disparos de T2}
		if (t<3*1.6666666666e-3) then
		begin
			trazo(18,-rf[1]);
		end;

		if (t>9*1.66666666e-3) then
		begin
			trazo(21,-rf[1]);
		end;


		{los 2 siguientes corresponden a disparos de T3}
		if (t<5*1.6666666666e-3) then
		begin
			trazo(19,rf[3]);
		end;

		if (t>11*1.66666666e-3) then
		begin
			trazo(22,rf[3]);
		end;

	trazo(2,uref);			{uref para disparo arco coseno}


	if t >= tfinal then
	begin
	{llegamos al final de la simulacin}

		for k:= 0 to 22 do
		begin
			borrecanal(k,0);
			desligue(k);
			borde;

			if (k<>1) and (k<>6) and (k<>7) and (k<>8) and (k<>9) and (k<>10) then
			begin
				grid;
				linea0;
			end
			else
			begin
				gridy:=13;
				grid;
				gridy:=grillay;
			end;
		end;

		{nueva simulacin}
		t:= t-tfinal+tinicial;


	end;



end;


(*********************************************************************
 Principal
*********************************************************************)
BEGIN
	quita:=5;			{separaciones de pantallas}


	{decimos a simeep donde estan nuestras funciones}
	FilUSR.Cirt1:=Cirt1;
	FilUSR.Cirt2:=Cirt2;
	FilUSR.FuncsR:=FuncsR;
	FilUSR.Inicio:=Inicio;
	FilUSR.Pesca:=Pesca;
	FilUSR.Fin:=Fin;

	{definicin de controles}
	UCntrl.init;

	UCntrl.insertdisplay(new(PCtrlVerTexto, init
	{ rnglon, texto1  ,            texto2,             texto3}
	(	0,	'Puente de tiristores','con iL=Id y comando','Arco-coseno.')));

	UCntrl.insert(new(PCtrlBotonVar,Init
	{renglon, nombre,   unidad,decimales, min, max,   def, inc,    var}
	(  8,   '  U ref.',  'mV'  ,0        ,   -990, 990,    870 ,  30, uref)));

	UCntrl.insertdisplay(new(PCtrlVer2Var,init
	{ renglon, nombre1, uni1,deci1,var1 ,nombre2, uni2,deci2,var2}
	(       7, ' Disparo',  '',  1  ,alpha ,'   wt', '',  1  ,wt)));

	paseo;												  {simulacin}

end.
