unit uActorNodal;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  uCosa, uGlobs, uActores, uFechas, unodos, uCosaConNombre, uFichasLPD,
  xMatDefs, Classes, contnrs, uauxiliares;

type

  { TActorNodal }

  TActorNodal = class(TActor)

    function ReferenciaAlNodo(nodo: TNodo): boolean; virtual; abstract;

  end;

  { TActorUniNodal }

  TActorUniNodal = class(TActorNodal)

  public
    P: TDAOfNReal; // Inyectada como generador al nodo

    (**************************************************************************)
    (*               A T R I B U T O S   P E R S I S T E N T E S              *)
    (**************************************************************************)

    Nodo: TNodo;
    barras_flucar: TDAOfNint;
    codigos_flucar: TDAOfString;
    (**************************************************************************)

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades: TFichasLPD; Nodo: TNodo);

    function Rec: TCosa_RecLnk; override;
    procedure BeforeRead(version, id_hilo: integer); override;
    procedure AfterRead(version, id_hilo: integer); override;

    function ReferenciaAlNodo(nodo: TNodo): boolean; override;
    procedure PubliVars; override;
    procedure PrepararMemoria(Catalogo: TCatalogoReferencias; globs: TGlobs); override;
    procedure Free; override;
    {$IFDEF BOSTA}
    procedure AfterInstantiation; override;
    {$ENDIF}
  end;

  { TActorBiNodal }

  TActorBiNodal = class(TActorNodal)
  public
    (**************************************************************************)
    (*               A T R I B U T O S   P E R S I S T E N T E S              *)
    (**************************************************************************)

    NodoA, NodoB: TNodo;
    (**************************************************************************)

    // resultados del despacho
    P_NodoA, P_NodoB: TDAOfNReal; //[MW] Potencia inyectada como generador en cada Nodo.

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades: TFichasLPD; NodoA, NodoB: TNodo);

    function Rec: TCosa_RecLnk; override;
    procedure BeforeRead(version, id_hilo: integer); override;
    procedure AfterRead(version, id_hilo: integer); override;

    function ReferenciaAlNodo(nodo: TNodo): boolean; override;
    procedure PrepararMemoria(Catalogo: TCatalogoReferencias; globs: TGlobs); override;
    procedure Free; override;

  end;

procedure AlInicio;
procedure ALFinal;

implementation

//--------------------------
// Mtodos de TActorUniNodal
//==========================

constructor TActorUniNodal.Create(capa: integer; nombre: string;
  nacimiento, muerte: TFecha; lpdUnidades: TFichasLPD; Nodo: TNodo);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades);
  self.Nodo := Nodo;
  barras_flucar := nil;
  codigos_flucar := nil;
end;

function TActorUniNodal.Rec: TCosa_RecLnk;
begin
  Result:=inherited Rec;
  Result.addCampoDef_ref('nodo', TCosa(self.Nodo), Self);
  Result.addCampoDef('barras_flucar', barras_flucar, 49);
  Result.addCampoDef('codigos_flucar', codigos_flucar, 49);
end;

procedure TActorUniNodal.BeforeRead(version, id_hilo: integer);
begin
  inherited BeforeRead(version, id_hilo);
end;

procedure TActorUniNodal.AfterRead(version, id_hilo: integer);
begin
  inherited AfterRead(version, id_hilo);
  if Version >= 49 then
    clear_vacias(codigos_flucar);
  self.Nodo := nil;
end;

function TActorUniNodal.ReferenciaAlNodo(nodo: TNodo): boolean;
begin
  Result := self.nodo = nodo;
end;

procedure TActorUniNodal.PubliVars;
begin
  inherited PubliVars;
  PublicarVariableVR('P', '[MW]', 6, 2, P, True, True);
end;

procedure TActorUniNodal.PrepararMemoria(Catalogo: TCatalogoReferencias; globs: TGlobs);
begin
  inherited PrepararMemoria(Catalogo, globs);
  setlength(P, globs.NPostes);
end;


procedure TActorUniNodal.Free;
begin
  setlength(P, 0);
  inherited Free;
end;


{$IFDEF BOSTA}
procedure TActorUniNodal.AfterInstantiation;
begin

  inherited AfterInstantiation;
  clear_vacias(codigos_flucar);
  self.Nodo := nil;

end;
{$ENDIF}

//-------------------------
// Mtodos de TActorBiNodal
//=========================

constructor TActorBiNodal.Create(capa: integer; nombre: string;
  nacimiento, muerte: TFecha; lpdUnidades: TFichasLPD; NodoA, NodoB: TNodo);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades);
  self.NodoA := NodoA;
  self.NodoB := NodoB;
end;

function TActorBiNodal.Rec: TCosa_RecLnk;
begin
  Result:=inherited Rec;
  Result.addCampoDef_ref('nodoA', TCosa(NodoA), Self);
  Result.addCampoDef_ref('nodoB', TCosa(NodoB), Self);
end;

procedure TActorBiNodal.BeforeRead(version, id_hilo: integer);
begin
  inherited BeforeRead(version, id_hilo);
end;

procedure TActorBiNodal.AfterRead(version, id_hilo: integer);
begin
  inherited AfterRead(version, id_hilo);
end;


function TActorBiNodal.ReferenciaAlNodo(nodo: TNodo): boolean;
begin
  Result := (nodoA = nodo) or (nodoB = nodo);
end;

procedure TActorBiNodal.PrepararMemoria(Catalogo: TCatalogoReferencias; globs: TGlobs);




begin
  inherited prepararMemoria(Catalogo, globs);
  setlength(P_NodoA, globs.NPostes);
  setlength(P_NodoB, globs.NPostes);
end;

procedure TActorBiNodal.Free;
begin
  setlength(P_NodoA, 0);
  setlength(P_NodoB, 0);
  inherited Free;
end;

procedure AlInicio;
begin
  //  registrarClaseDeCosa(TActorUniNodal.ClassName, TActorUniNodal);
  //  registrarClaseDeCosa(TActorBiNodal.ClassName, TActorBiNodal);
end;

procedure AlFinal;
begin
end;

end.
